import {Injectable} from "@angular/core";
import axios from "axios";
import {environment} from "../../../environment/env";

@Injectable({
  providedIn: 'root'
})
export class ConformidadService {

  t_asistencia_rest_link = environment.t_asistencia_rest_link;
  t_horizon_rest_link = environment.horizon_services_link;
  t_facturacion_electronica_link = environment.facturacion_electronica_link;
  value_idDrive = environment.ID_CARPETA_DRIVE_LOGISTICA;
  value_correoDrive = environment.CORREO_DRIVE_LOGISTICA;
  t_redireccionamiento = environment.redireccionamiento;

  constructor() {
  }

  async listadoGeneral(opcion: number) {
    try {
      const respuesta = await axios.post(this.t_asistencia_rest_link + '/api/v1/Conformidad/listadoGeneral',
        {
          v_opcion: opcion,
        });
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async listadoGeneralHorizon(json: any) {

    const parametros =[
      json.accion || 0,
      json.nombreValor || '',
      json.sede || '',
    ]

    try {
      const respuesta = await axios.post(this.t_horizon_rest_link + '/api/procedure/Procedimiento',
        {
          database: "sqlserver",
          procedure :"USP_GENERAL_FORMULARIO",
          params : parametros
        },
        {
          headers: {
            'Content-Type': 'application/json'
          }
        }
      );
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async gestionConformidad(json: any) {

    const Parametros = {
      opcion : json.opcion || 0,
      codconformidad : json.codconformidad || 0,
      codper : json.codper || 0,
      codlocal : json.codlocal || '',
      fechainicio : json.fechainicio || '1999-01-01',
      fechafin : json.fechafin || '1999-01-01',
      descproblema : json.descproblema || '',
      desccausa : json.desccausa || '',
      codtipotrabajo: json.codtipotrabajo || 0 ,
      desctrabajo: json.desctrabajo || '',
      materialsobrate: json.materialsobrate || '',
      pasaje : json.pasaje || 0,
      codrequerimiento : json.codrequerimiento || 0
    };

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/registrarConformidad', Parametros);
      console.log(respuesta);
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }

  async subidaEvidencia(json:any){
    const url = `${this.t_redireccionamiento}/google/drive`;

    const formData = new FormData();
    formData.append('archivo_nombre', json.nombreArchivo);
    formData.append('archivo_binario', json.archivo);
    formData.append('archivo_id_actual', '');
    formData.append('carpeta_id', this.value_idDrive);
    formData.append('correo', this.value_correoDrive);
    formData.append('app_nombre', 'EVIDENCIAS_CONFORMIDAD');
    formData.append('carpeta_nombre', 'EVIDENCIA1');


    const controller = new AbortController();
    const timeout = setTimeout(() => controller.abort(), 30000);  // 30 segundos

    try {
      const response = await fetch(url, {
        method: 'POST',
        body: formData,
        signal: controller.signal,  // 30 segundos de timeout
      });

      const statusCode = response.status;
      const responseBody = await response.json();

      // Construir respuesta en formato JSON
      /*const respuesta = {
        statusCode: statusCode,
        responseBody: responseBody,
      };
      */
      clearTimeout(timeout);  // Limpiar, REVISA SI ESTO ESTA BIEN O QUITARLO

      let respuesta :any ;

      //console.log(statusCode,responseBody);
      if(statusCode === 200){

        //console.log(responseBody);

        const registroEvidencia = await this.registroDocumento({
          codconformidad:  json.codconformidad || 0,
          tipoEvidencia : json.tipoEvidencia || 0,
          idDrive : responseBody.data.archivo_id || '',
          enlaceDrive : responseBody.data.archivo_url_vista || '',
          enlaceDescarga : responseBody.data.archivo_url || '',
          nombreDocumento : json.nombreArchivo || ''
        });

        if(registroEvidencia?.status){
          respuesta = {status: true, message: "Imagen subida"};
        }else{
          respuesta = { status :false , message : "Imagen no subida" };
        }


        //respuesta = { status :true , message : "Imagen subida" };

        /*const respuestaDocumento = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/registroDocumentos', registroEvidencia);+
        console.log(respuestaDocumento);
        respuesta =  respuestaDocumento;*/

      }else{
        respuesta = { status :false , message : "Imagen no subida" };
      }

      return respuesta;
    } catch (error) {
      console.error('Error al registrar el archivo:', error);
      throw error;
    }
  }

  async registroDocumento(json : any){

    try {
      const respuesta = await axios.post(this.t_facturacion_electronica_link + '/api/v1/conformidad/registroDocumentos', json);
      //console.log(respuesta);
      return respuesta.data;
    } catch (e) {
      return e;
    }

  }


  /*async listadoRequerimientos(json: any) {
    const parametros =[
      json.accion || 0,
      json.nombreValor || '',
      json.sede || '',
    ]

    try {
      const respuesta = await axios.post(this.t_horizon_rest_link + '/api/procedure/Procedimiento',
        {
          database: "sqlserver",
          procedure :"USP_GENERAL_FORMULARIO",
          params : parametros
        },
        {
          headers: {
            'Content-Type': 'application/json'
          }
        });
      return respuesta.data;
    } catch (e) {
      return e;
    }
  }*/


}
